<?php

namespace Drupal\paragraph_blocks;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\path_alias\AliasManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class ParagraphBlocksEntityManager.
 */
class ParagraphBlocksEntityManager implements ContainerInjectionInterface {

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected AliasManagerInterface $aliasManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * ParagraphBlocksEntityManager constructor.
   */
  public function __construct(RequestStack $request_stack) {
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): ParagraphBlocksEntityManager {
    return new static(
      $container->get('request_stack')
    );
  }

  /**
   * Return the entity from Layout Builders section storage in request.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity of the referer, or null if none.
   *
   * @throws \Drupal\Component\Plugin\Exception\ContextException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getRefererEntity() {
    /** @var \Drupal\layout_builder\Plugin\SectionStorage\OverridesSectionStorage $section_storage */
    $section_storage = $this->requestStack->getCurrentRequest()->attributes->get('section_storage');

    // If section storage is for a single entity override there will be a entity to return.
    if ('overrides' === $section_storage->getPluginId()) {
      $entity = $section_storage->getContext('entity')
        ->getContextData()
        ->getEntity();
      if (\Drupal::moduleHandler()->moduleExists('content_moderation')) {
        return $this->getLatestEntityRevision($entity);
      }
      return $entity;
    }
    return NULL;
  }

  /**
   * Load the latest entity revision when using content moderation.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *
   * @return \Drupal\Core\Entity\ContentEntityInterface
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getLatestEntityRevision(ContentEntityInterface &$entity): ContentEntityInterface {
    /** @var \Drupal\content_moderation\ModerationInformation $moderation_info */
    $moderation_info = \Drupal::service('content_moderation.moderation_information');
    $pending = $moderation_info->hasPendingRevision($entity);
    if ($pending) {
      /** @var \Drupal\Core\Entity\ContentEntityStorageInterface $storage */
      $entity_type_id = $entity->getEntityTypeId();
      $storage = \Drupal::entityTypeManager()
        ->getStorage($entity_type_id);
      $latest_revision_id = $storage->getLatestTranslationAffectedRevisionId($entity->id(), $entity->language()
        ->getId());

      $entity = \Drupal::entityTypeManager()
        ->getStorage($entity_type_id)
        ->loadRevision($latest_revision_id);
    }

    return $entity;
  }

}
